#!/usr/bin/env python
#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
# 
#   http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
#

import qpid
import sys
import os
from qpid.util import connect
from qpid.connection import Connection
from qpid.datatypes import Message, RangedSet, uuid4
from qpid.queue import Empty
from qpid.session import SessionException
from math import fabs
from threading import Thread

class Publisher:
    def __init__(self, host, port):
        self.connection = Connection(connect(host, port))
        self.connection.start()
        self.session = self.connection.session(str(uuid4()))

    def run(self):
        props = self.session.delivery_properties(routing_key=self.queue)
        for i in range(self.count):
            self.session.message_transfer(message=Message(props, str(i+1)))

    def publish(self, queue, count):
        self.queue = queue
        self.count = count
        self.thread = Thread(target = self.run)
        self.thread.start()

    def close(self):
        if (self.thread): self.thread.join()
        self.session.close()
        self.connection.close()

class Receiver:
    def __init__(self, host, port, queue):
        self.queue = queue
        self.connection = Connection(connect(host, port))
        self.connection.start()
        self.session = self.connection.session(str(uuid4()))
        self.session.queue_declare(queue=self.queue)

    def process(self, count):    
        self.session.message_subscribe(queue=self.queue, destination="received")
        incoming = self.session.incoming("received")
        incoming.start()
        for i in range(count):
            incoming.get(timeout=1)
        try:
            self.session.queue_declare(queue="i-don't-exist", passive=True)
        except SessionException, e: 
            self.session = self.connection.session(str(uuid4()))

    def close(self):
        self.session.close()
        self.connection.close()

#  Set parameters for login

host="127.0.0.1"
port=5672

if len(sys.argv) > 1 :
    host=sys.argv[1]
if len(sys.argv) > 2 :
    port=int(sys.argv[2])

#  Create a connection.

queue = "bz461816"
publisher = Publisher(host, port)
receiver = Receiver(host, port, queue)

publisher.publish(queue, 1000)

for i in range(100):
    print "%d of %d" % ((i+1), 100)
    receiver.process(10)

publisher.close()
receiver.close()

# eof
