#!/bin/bash

# Copyright (c) 2008 Red Hat, Inc. All rights reserved. This copyrighted material 
# is made available to anyone wishing to use, modify, copy, or
# redistribute it subject to the terms and conditions of the GNU General
# Public License v.2.
#
# This program is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
# PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# Author: Frantisek Reznicek <freznice@redhat.com>

# test whether MRG test common code installed, try to load if possible
# ###########################################################################
mrg_test_config_fn="${HOME}/.mrg_test_config"

if [ -r ${mrg_test_config_fn} -a -s ${mrg_test_config_fn} ]; then
  # load in MRG test common configuration
  source ${mrg_test_config_fn}
fi

if [[ -r ${RHTS_TEST_MRG_COMMON_BASH_FUNCTIONS} && -s ${RHTS_TEST_MRG_COMMON_BASH_FUNCTIONS} && -d ${RHTS_TEST_MRG_COMMON_DIR} ]]; then
  echo "INFO:MRG test common package qpid_test_common_mrg_install installed OK"
else
  rierr_cnt=0
  # MRG common code not installed - installation needed
  echo "MRG test common stuff not installed ... trying to recover (make sure you run qpid_test_common_mrg_install before this test)"
  mrg_test_common_pkg_name="qpid_test_common_mrg_install"
  mrg_test_common_tgz_url="http://cvs.devel.redhat.com/cgi-bin/cvsweb.cgi/tests/distribution/MRG_Messaging/qpid_test_common_mrg_install/qpid_test_common_mrg_install.tar.gz?tarball=1"
  # getting package using wget (avail on all RHELs)
  wget -q ${mrg_test_common_tgz_url} -O ${mrg_test_common_pkg_name}.tar.gz
  if [ "$?" != "0" ]; then
    let "rierr_cnt++"
  fi
  # extracting the package
  tar xzf ${mrg_test_common_pkg_name}.tar.gz
  if [ -d  ./${mrg_test_common_pkg_name} -a -r ./${mrg_test_common_pkg_name}/runtest.sh  ]; then
    # moving to downloaded package
    pushd ${mrg_test_common_pkg_name} > /dev/null
    # executing MRG common test procedure
    bash ./runtest.sh
    if [ "$?" == "0" -a -r ${mrg_test_config_fn} -a -s ${mrg_test_config_fn} ]; then
      source ${mrg_test_config_fn}
    else  
      let "rierr_cnt++"
    fi
    popd > /dev/null
  else
    let "rierr_cnt++"
  fi
  # verify the MRG test configuration
  if [[ -r ${RHTS_TEST_MRG_COMMON_BASH_FUNCTIONS} && -s ${RHTS_TEST_MRG_COMMON_BASH_FUNCTIONS} && -d ${RHTS_TEST_MRG_COMMON_DIR} && "${rierr_cnt}" == "0" ]]; then
    echo "INFO:MRG test common package qpid_test_common_mrg_install installed OK"
  else
    echo "ERROR:MRG test common package qpid_test_common_mrg_install failed to download and/or install! (err_cnt:${rierr_cnt})"
    rm -rf ${mrg_test_common_pkg_name}*
    exit 1
  fi  
fi

# source used MRG test common functions
# ###########################################################################
source ${RHTS_TEST_MRG_COMMON_BASH_FUNCTIONS}

# global test variables 
# ###########################################################################
VERSION=1.0                  # test version
RESULT=FAIL                  # FAIL unless test explicitly passes
ERR_CNT=0                    # error counter init
TST_CNT=0                    # test count
OUTPUTFILE="$PWD/$(basename $PWD).log"
VERBOSE_LEVEL=${TEST_PARAM_RHTS_TEST_VERBOSE_LEVEL:-1}

# Source RHTS environment (. /usr/bin/rhts-environment.sh)
# ###########################################################################
rhts_load_environment


# other variables
# ###########################################################################

DATE_TIME_START=$(date +%Y%m%d_%H%M%S)
TEMP_FILE=tmp_${DATE_TIME_START}
TEMP_FILE_GLOB=tmp_*

# Qpid info
TESTSUITE_DIR="`dirname $0`"
QPIDD_DATADIR="${PWD}/rhts_qpidd/${DATE_TIME_START}"

# use TEST_PARAM_RHTS_TEST_QPIDD_DIR (RHTS_TEST_QPIDD_DIR) for overriding std paths
# ###########################################################################
#TEST_PARAM_RHTS_TEST_QPIDD_DIR=/home/freznice/qpidd_compiled/sbin/
PREFIX_DIR=/usr/bin
QPIDD_DIR=${TEST_PARAM_RHTS_TEST_QPIDD_DIR:-/usr/sbin}

# set qpidd params
export QPIDD_PORT=5672
QPIDD_TEST_PARAMS="--auth no --log-enable info+"
QPIDD_TEST_PARAMS="--auth no --log-enable trace+"
QPIDD_TRANSCRIPT="qpidd.transcript.log"
QPIDD_LOCK=qpidd.lock
QPIDD_ECODE=qpidd.ecode

COMMON_TRANSCRIPT="common_transcript.log"

# ###########################################################################
# main - testing sequence starts here
# ###########################################################################

# ${OUTPUTFILE} logging starts here
{

# install additional packages on top of the default ones
# ###########################################################################
pkgs2install=""
install_packages_yum ${pkgs2install}

# gather additional info
# ###########################################################################
if [ -n "${VERBOSE_LEVEL}" -a "${VERBOSE_LEVEL}" != "0" ]; then
  # basic info
  get_essential_info
  
  # get info about newly installed packages by this test (diff view only)
  get_installed_packages ${RHTS_TEST_MRG_INSTALLED_PACKAGES}
fi


# test additional executables which need to be installed 
# ###########################################################################
tools_to_check=""
check_tool_install ${tools_to_check}
if [ "$?" != "0" ]; then
  rhts_report_result ${TEST} ${RESULT} 1
  exit 1
fi


# MRG components tests
# ###########################################################################
if [ ! -x  ${QPIDD_DIR}/qpidd ]; then
  which qpidd >${TEMP_FILE} 2> /dev/null
  if [ "$?" != "0" ]; then
    lognl "ERROR:qpidd not executable"
    rhts_report_result ${TEST} ${RESULT} 1
    exit 1
  else
    QPIDD_DIR=$(dirname $(cat ${TEMP_FILE}))
  fi
fi


# get tools version information
lognl "Specific tools version info:"
TOOL_LIST="${QPIDD_DIR}/qpidd"
for i_tool in ${TOOL_LIST}; do
  lognl "  ${i_tool}: ${QPIDD_DIR}/qpidd  $(${i_tool} --version | head -1)"
done
lognl "  qpidd path: ${QPIDD_DIR}/qpidd"


lognl "========================TEST started========================"


echo "Test $(basename ${PWD}): PART A" > ${COMMON_TRANSCRIPT}
lognl "-PART A-----------------------------------------------------"

NR_OF_LOOPS=3


# stop all qpidd brokers
# ---------------------------------------------------------------------------
lognl "Stopping all running qpidd instances..."
qpidd_stop
ecode="$?"
if [ "${ecode}" == "0" -a "${QPIDD_INST_CNT}" == "0" ]; then
  lognl ".qpidd stopped ok"
else
  lognl ".ERROR:qpidd didn't stop (ecode:${ecode}|qpidd_int_cnt:${QPIDD_INST_CNT})"
  let "ERR_CNT++"
fi


# main loop
for ((i_loop=0; i_loop<${NR_OF_LOOPS}; i_loop++)); do
  
  let "i_loop_p1=${i_loop} + 1"
  lognl_ct "run(A) ${i_loop_p1}/${NR_OF_LOOPS}--------------------------------[err_cnt:${ERR_CNT}]-"

  # create qpidd data journal storage
  # -------------------------------------------------------------------------
  trasher ${QPIDD_DATADIR}
  mkdir -p ${QPIDD_DATADIR}
  QPIDD_DATADIR_JOURNAL=$(mktemp -d ${QPIDD_DATADIR}/broker.XXXX)
  
  # start qpidd
  # -------------------------------------------------------------------------
  qpidd_start_normal ${QPIDD_PORT} ${QPIDD_DATADIR_JOURNAL} ${QPIDD_TRANSCRIPT} ${QPIDD_TEST_PARAMS}
  
  # wait for qpidd ready
  # -------------------------------------------------------------------------
  qpidd_wait_on_settle ${QPIDD_TRANSCRIPT} "Listening on TCP port" ${QPIDD_LOCK} 20
  if [ "$?" == "0" ]; then
    lognl ".qpidd settled"
  else
    lognl ".ERROR:qpidd not started"
    let "ERR_CNT++"
  fi


  # running the core python test
  # -------------------------------------------------------------------------
  lognl ".running core test (log:${COMMON_TRANSCRIPT})"
  python ./bz461816.py >> ${COMMON_TRANSCRIPT} 2>&1
  ecode="$?"
  let "TST_CNT++"
  
  if [ "${ecode}" != "0" ]; then
    lognl "..ERROR:core test failed! (ecode:${ecode})"
    let "ERR_CNT++"
  else
    lognl "..core test passed"
  fi
  
  
  # verify qpidd is up and running
  # -------------------------------------------------------------------------
  lognl ".qpidd status check"
  qpidd_status
  if [ -e ${QPIDD_LOCK} -a \
       "${QPIDD_INST_CNT}" == "1" -a \
       "${QPIDD_INST_PORT}" == "${QPIDD_INST_PORT}" ]; then
    lognl "..qpidd running fine on port ${QPIDD_INST_PORT} (${QPIDD_INST_CNT} instance[s])"
  else
    lognl "..ERROR:qpidd not running fine! (pid[s]:${QPIDD_INST_PID}, port[s]:${QPIDD_INST_PORT}, inst_cnt:${QPIDD_INST_CNT})"
    let "ERR_CNT++"
  fi
  
  # stopping qpidd
  # -------------------------------------------------------------------------
  lognl ".qpidd stop"
  qpidd_stop
  ecode="$?"
  if [ "${ecode}" == "0" -a "${QPIDD_INST_CNT}" == "0" ]; then
    lognl ".qpidd stopped ok"
  else
    lognl ".ERROR:qpidd didn't stop (ecode:${ecode}|qpidd_int_cnt:${QPIDD_INST_CNT})"
    let "ERR_CNT++"
  fi
  
  # remove qpidd lock
  # -------------------------------------------------------------------------
  lognl ".remove qpidd parent lock (if needed)..."
  if [ -e ${QPIDD_LOCK} ]; then
    rm -f ${QPIDD_LOCK}
  fi
  

  # waiting & logging
  # -------------------------------------------------------------------------
  cat ${QPIDD_TRANSCRIPT} >> ${COMMON_TRANSCRIPT}
  sleep 1
  
done

#lognl "-PART B-----------------------------------------------------"



lognl "========================TEST finished======================="

# ###########################################################################
# report results
# ###########################################################################

if [ ${ERR_CNT} -eq 0 ]; then
  RESULT=PASS
fi

if [ -n "${TESTID}" ]; then
  rhts_submit_log -S ${RESULT_SERVER} -T ${TESTID} -l ${COMMON_TRANSCRIPT}
fi

rhts_report_result ${TEST} ${RESULT} ${ERR_CNT}

} 2>&1 | tee ${OUTPUTFILE}

# clean-up files / dirs
del_list="${TEMP_FILE_GLOB} ${QPIDD_LOCK} ${QPIDD_ECODE}"
trasher ${del_list} 

# ###########################################################################
# end - testing sequence stops here
# ###########################################################################
Exit

# ###########################################################################
# eof
# ###########################################################################
